@@ -12,16 +12,16 @@ import sys
 import copy
 
 
-from ruamel_yaml.compat import ordereddict, PY2, string_types
+from ruamel_yaml.compat import ordereddict, PY2, string_types, MutableSliceableSequence
 from ruamel_yaml.scalarstring import ScalarString
 
 if PY2:
-    from collections import MutableSet, Sized, Set, MutableMapping
+    from collections import MutableSet, Sized, Set, MutableMapping, Mapping
 else:
-    from collections.abc import MutableSet, Sized, Set, MutableMapping
+    from collections.abc import MutableSet, Sized, Set, MutableMapping, Mapping
 
 if False:  # MYPY
-    from typing import Any, Dict, Optional, List, Union, Optional  # NOQA
+    from typing import Any, Dict, Optional, List, Union, Optional, Iterator  # NOQA
 
 # fmt: off
 __all__ = ["CommentedSeq", "CommentedKeySeq",
@@ -297,7 +297,10 @@ class CommentedBase(object):
         from .error import CommentMark
 
         if column is None:
-            column = self._yaml_get_column(key)
+            try:
+                column = self._yaml_get_column(key)
+            except AttributeError:
+                column = 0
         if comment[0] != '#':
             comment = '# ' + comment
         if column is None:
@@ -382,8 +385,58 @@ class CommentedBase(object):
         raise NotImplementedError
 
 
-class CommentedSeq(list, CommentedBase):
-    __slots__ = (Comment.attrib,)
+class CommentedSeq(MutableSliceableSequence, CommentedBase):
+    __slots__ = (Comment.attrib, '_lst')
+
+    def __init__(self, *args, **kw):
+        # type: (Any, Any) -> None
+        self._lst = list(*args, **kw)
+
+    def __getsingleitem__(self, idx):
+        # type: (Any) -> Any
+        return self._lst[idx]
+
+    def __setsingleitem__(self, idx, value):
+        # type: (Any, Any) -> None
+        # try to preserve the scalarstring type if setting an existing key to a new value
+        if idx < len(self):
+            if (
+                isinstance(value, string_types)
+                and not isinstance(value, ScalarString)
+                and isinstance(self[idx], ScalarString)
+            ):
+                value = type(self[idx])(value)
+        self._lst.__setitem__(idx, value)
+
+    def __delsingleitem__(self, idx=None):
+        # type: (Any) -> Any
+        del self._lst[idx]
+        self.ca.items.pop(idx, None)  # might not be there -> default value
+        for list_index in sorted(self.ca.items):
+            if list_index < idx:
+                continue
+            self.ca.items[list_index - 1] = self.ca.items.pop(list_index)
+
+    def __len__(self):
+        # type: () -> int
+        return len(self._lst)
+
+    def insert(self, idx, val):
+        # type: (Any, Any) -> None
+        """the comments after the insertion have to move forward"""
+        self._lst.insert(idx, val)
+        for list_index in sorted(self.ca.items, reverse=True):
+            if list_index < idx:
+                break
+            self.ca.items[list_index + 1] = self.ca.items.pop(list_index)
+
+    def extend(self, val):
+        # type: (Any) -> None
+        self._lst.extend(val)
+
+    def __eq__(self, other):
+        # type: (Any) -> bool
+        return bool(self._lst == other)
 
     def _yaml_add_comment(self, comment, key=NoComment):
         # type: (Any, Optional[Any]) -> None
@@ -400,25 +453,6 @@ class CommentedSeq(list, CommentedBase):
         # type: (Any) -> Any
         return self.ca.items[key][0].start_mark.column
 
-    def insert(self, idx, val):
-        # type: (Any, Any) -> None
-        """the comments after the insertion have to move forward"""
-        list.insert(self, idx, val)
-        for list_index in sorted(self.ca.items, reverse=True):
-            if list_index < idx:
-                break
-            self.ca.items[list_index + 1] = self.ca.items.pop(list_index)
-
-    def pop(self, idx=None):
-        # type: (Any) -> Any
-        res = list.pop(self, idx)
-        self.ca.items.pop(idx, None)  # might not be there -> default value
-        for list_index in sorted(self.ca.items):
-            if list_index < idx:
-                continue
-            self.ca.items[list_index - 1] = self.ca.items.pop(list_index)
-        return res
-
     def _yaml_get_column(self, key):
         # type: (Any) -> Any
         column = None
@@ -458,17 +492,24 @@ class CommentedSeq(list, CommentedBase):
             self.copy_attributes(res, deep=True)
         return res
 
-    def __setitem__(self, idx, value):
-        # type: (Any, Any) -> None
-        # try to preserve the scalarstring type if setting an existing key to a new value
-        if idx < len(self):
-            if (
-                isinstance(value, string_types)
-                and not isinstance(value, ScalarString)
-                and isinstance(self[idx], ScalarString)
-            ):
-                value = type(self[idx])(value)
-        list.__setitem__(self, idx, value)
+    def __add__(self, other):
+        # type: (Any) -> Any
+        return self._lst + other
+
+    def sort(self):
+        # type: () -> None
+        tmp_lst = sorted(zip(self._lst, range(len(self._lst))))
+        self._lst = [x[0] for x in tmp_lst]
+        itm = self.ca.items
+        self.ca._items = {}
+        for idx, x in enumerate(tmp_lst):
+            old_index = x[1]
+            if old_index in itm:
+                self.ca.items[idx] = itm[old_index]
+
+    def __repr__(self):
+        # type: () -> Any
+        return self._lst.__repr__()
 
 
 class CommentedKeySeq(tuple, CommentedBase):
@@ -656,7 +697,7 @@ class CommentedMap(CommentedBase, MutableMapping):
             self.ca.comment[1] = pre_comments
         return pre_comments
 
-    def update(self, vals):
+    def update(self, vals):  # type: ignore
         # type: (Any) -> None
         try:
             self._od.update(vals)
@@ -943,11 +984,111 @@ class CommentedMap(CommentedBase, MutableMapping):
         return res
 
 
+# based on brownie mappings
+@classmethod  # type: ignore
+def raise_immutable(cls, *args, **kwargs):
+    # type: (Any, *Any, **Any) -> None
+    raise TypeError('{} objects are immutable'.format(cls.__name__))
+
+
+class CommentedKeyMap(CommentedBase, Mapping):
+    __slots__ = Comment.attrib, '_od'
+    """This primarily exists to be able to roundtrip keys that are mappings"""
+
+    def __init__(self, *args, **kw):
+        # type: (Any, Any) -> None
+        if hasattr(self, '_od'):
+            raise_immutable(self)
+        try:
+            self._od = ordereddict(*args, **kw)
+        except TypeError:
+            if PY2:
+                self._od = ordereddict(args[0].items())
+            else:
+                raise
+
+    __delitem__ = __setitem__ = clear = pop = popitem = setdefault = update = raise_immutable
+
+    # need to implement __getitem__, __iter__ and __len__
+    def __getitem__(self, index):
+        # type: (Any) -> Any
+        return self._od[index]
+
+    def __iter__(self):
+        # type: () -> Iterator[Any]
+        for x in self._od.__iter__():
+            yield x
+
+    def __len__(self):
+        # type: () -> int
+        return len(self._od)
+
+    def __hash__(self):
+        # type: () -> Any
+        return hash(tuple(self.items()))
+
+    def __repr__(self):
+        # type: () -> Any
+        if not hasattr(self, merge_attrib):
+            return self._od.__repr__()
+        return 'ordereddict(' + repr(list(self._od.items())) + ')'
+
+    @classmethod
+    def fromkeys(keys, v=None):
+        # type: (Any, Any) -> Any
+        return CommentedKeyMap(dict.fromkeys(keys, v))
+
+    def _yaml_add_comment(self, comment, key=NoComment):
+        # type: (Any, Optional[Any]) -> None
+        if key is not NoComment:
+            self.yaml_key_comment_extend(key, comment)
+        else:
+            self.ca.comment = comment
+
+    def _yaml_add_eol_comment(self, comment, key):
+        # type: (Any, Any) -> None
+        self._yaml_add_comment(comment, key=key)
+
+    def _yaml_get_columnX(self, key):
+        # type: (Any) -> Any
+        return self.ca.items[key][0].start_mark.column
+
+    def _yaml_get_column(self, key):
+        # type: (Any) -> Any
+        column = None
+        sel_idx = None
+        pre, post = key - 1, key + 1
+        if pre in self.ca.items:
+            sel_idx = pre
+        elif post in self.ca.items:
+            sel_idx = post
+        else:
+            # self.ca.items is not ordered
+            for row_idx, _k1 in enumerate(self):
+                if row_idx >= key:
+                    break
+                if row_idx not in self.ca.items:
+                    continue
+                sel_idx = row_idx
+        if sel_idx is not None:
+            column = self._yaml_get_columnX(sel_idx)
+        return column
+
+    def _yaml_get_pre_comment(self):
+        # type: () -> Any
+        pre_comments = []  # type: List[Any]
+        if self.ca.comment is None:
+            self.ca.comment = [None, pre_comments]
+        else:
+            self.ca.comment[1] = pre_comments
+        return pre_comments
+
+
 class CommentedOrderedMap(CommentedMap):
     __slots__ = (Comment.attrib,)
 
 
-class CommentedSet(MutableSet, CommentedMap):  # NOQA
+class CommentedSet(MutableSet, CommentedBase):  # NOQA
     __slots__ = Comment.attrib, 'odict'
 
     def __init__(self, values=None):
@@ -957,6 +1098,22 @@ class CommentedSet(MutableSet, CommentedMap):  # NOQA
         if values is not None:
             self |= values  # type: ignore
 
+    def _yaml_add_comment(self, comment, key=NoComment, value=NoComment):
+        # type: (Any, Optional[Any], Optional[Any]) -> None
+        """values is set to key to indicate a value attachment of comment"""
+        if key is not NoComment:
+            self.yaml_key_comment_extend(key, comment)
+            return
+        if value is not NoComment:
+            self.yaml_value_comment_extend(value, comment)
+        else:
+            self.ca.comment = comment
+
+    def _yaml_add_eol_comment(self, comment, key):
+        # type: (Any, Any) -> None
+        """add on the value line, with value specified by the key"""
+        self._yaml_add_comment(comment, value=key)
+
     def add(self, value):
         # type: (Any) -> None
         """Add an element."""
@@ -1005,14 +1162,14 @@ def dump_comments(d, name="", sep='.', out=sys.stdout):
     """
     if isinstance(d, dict) and hasattr(d, 'ca'):
         if name:
-            print(name)
-        print(d.ca, file=out)  # type: ignore
+            sys.stdout.write('{}\n'.format(name))
+        out.write('{}\n'.format(d.ca))  # type: ignore
         for k in d:
             dump_comments(d[k], name=(name + sep + k) if name else k, sep=sep, out=out)
     elif isinstance(d, list) and hasattr(d, 'ca'):
         if name:
-            print(name)
-        print(d.ca, file=out)  # type: ignore
+            sys.stdout.write('{}\n'.format(name))
+        out.write('{}\n'.format(d.ca))  # type: ignore
         for idx, k in enumerate(d):
             dump_comments(
                 k, name=(name + sep + str(idx)) if name else str(idx), sep=sep, out=out
