@@ -7,9 +7,12 @@ from __future__ import print_function
 import sys
 import os
 import types
+from abc import abstractmethod
 
+# fmt: off
 if False:  # MYPY
-    from typing import Any, Dict, Optional, List, Union, BinaryIO, IO, Text, Tuple   # NOQA
+    from typing import Any, Dict, Optional, List, Union, BinaryIO, IO, Text, Tuple, Optional  # NOQA
+# fmt: on
 
 _DEFAULT_YAML_VERSION = (1, 2)
 
@@ -22,8 +25,9 @@ except:  # NOQA
         from ordereddict import OrderedDict  # type: ignore
     # to get the right name import ... as ordereddict doesn't do that
 
-    class ordereddict(OrderedDict):   # type: ignore
+    class ordereddict(OrderedDict):  # type: ignore
         if not hasattr(OrderedDict, 'insert'):
+
             def insert(self, pos, key, value):
                 # type: (int, Any, Any) -> None
                 if pos >= len(self):
@@ -44,6 +48,7 @@ PY3 = sys.version_info[0] == 3
 
 
 if PY3:
+
     def utf8(s):
         # type: (str) -> str
         return s
@@ -56,7 +61,9 @@ if PY3:
         # type: (str) -> str
         return s
 
+
 else:
+
     def utf8(s):
         # type: (unicode) -> str
         return s.encode('utf-8')
@@ -67,7 +74,8 @@ else:
 
     def to_unicode(s):
         # type: (str) -> unicode
-        return unicode(s)    # NOQA
+        return unicode(s)  # NOQA
+
 
 if PY3:
     string_types = str
@@ -79,26 +87,31 @@ if PY3:
     MAXSIZE = sys.maxsize
     unichr = chr
     import io
+
     StringIO = io.StringIO
     BytesIO = io.BytesIO
     # have unlimited precision
     no_limit_int = int
+    from collections.abc import Hashable, MutableSequence, MutableMapping, Mapping  # NOQA
 
 else:
-    string_types = basestring    # NOQA
+    string_types = basestring  # NOQA
     integer_types = (int, long)  # NOQA
     class_types = (type, types.ClassType)
-    text_type = unicode          # NOQA
+    text_type = unicode  # NOQA
     binary_type = str
 
     # to allow importing
     unichr = unichr  # type: ignore
     from StringIO import StringIO as _StringIO
+
     StringIO = _StringIO
     import cStringIO
+
     BytesIO = cStringIO.StringIO
     # have unlimited precision
     no_limit_int = long  # NOQA not available on Python 3
+    from collections import Hashable, MutableSequence, MutableMapping, Mapping  # NOQA
 
 if False:  # MYPY
     # StreamType = Union[BinaryIO, IO[str], IO[unicode],  StringIO]
@@ -118,7 +131,7 @@ UNICODE_SIZE = 4 if sys.maxunicode > 65535 else 2
 def with_metaclass(meta, *bases):
     # type: (Any, Any) -> Any
     """Create a base class with a metaclass."""
-    return meta("NewBase", bases, {})
+    return meta('NewBase', bases, {})
 
 
 DBG_TOKEN = 1
@@ -126,7 +139,7 @@ DBG_EVENT = 2
 DBG_NODE = 4
 
 
-_debug = None  # type: Union[None, int]
+_debug = None  # type: Optional[int]
 if 'RUAMELDEBUG' in os.environ:
     _debugx = os.environ.get('RUAMELDEBUG')
     if _debugx is None:
@@ -136,6 +149,7 @@ if 'RUAMELDEBUG' in os.environ:
 
 
 if bool(_debug):
+
     class ObjectCounter(object):
         def __init__(self):
             # type: () -> None
@@ -148,7 +162,7 @@ if bool(_debug):
         def dump(self):
             # type: () -> None
             for k in sorted(self.map):
-                print(k, '->', self.map[k])
+                sys.stdout.write('{} -> {}'.format(k, self.map[k]))
 
     object_counter = ObjectCounter()
 
@@ -172,7 +186,9 @@ def dbg(val=None):
 def nprint(*args, **kw):
     # type: (Any, Any) -> None
     if bool(dbg):
-        print(*args, **kw)
+        dbgprint = print  # to fool checking for print statements
+        dbgprint(*args, **kw)
+
 
 # char checkers following production rules
 
@@ -200,8 +216,69 @@ def check_anchorname_char(ch):
 def version_tnf(t1, t2=None):
     # type: (Any, Any) -> Any
     from ruamel_yaml import version_info  # NOQA
+
     if version_info < t1:
         return True
     if t2 is not None and version_info < t2:
         return None
     return False
+
+
+class MutableSliceableSequence(MutableSequence):
+    __slots__ = ()
+
+    def __getitem__(self, index):
+        # type: (Any) -> Any
+        if not isinstance(index, slice):
+            return self.__getsingleitem__(index)
+        return type(self)([self[i] for i in range(*index.indices(len(self)))])  # type: ignore
+
+    def __setitem__(self, index, value):
+        # type: (Any, Any) -> None
+        if not isinstance(index, slice):
+            return self.__setsingleitem__(index, value)
+        assert iter(value)
+        # nprint(index.start, index.stop, index.step, index.indices(len(self)))
+        if index.step is None:
+            del self[index.start : index.stop]
+            for elem in reversed(value):
+                self.insert(0 if index.start is None else index.start, elem)
+        else:
+            range_parms = index.indices(len(self))
+            nr_assigned_items = (range_parms[1] - range_parms[0] - 1) // range_parms[2] + 1
+            # need to test before changing, in case TypeError is caught
+            if nr_assigned_items < len(value):
+                raise TypeError(
+                    'too many elements in value {} < {}'.format(nr_assigned_items, len(value))
+                )
+            elif nr_assigned_items > len(value):
+                raise TypeError(
+                    'not enough elements in value {} > {}'.format(
+                        nr_assigned_items, len(value)
+                    )
+                )
+            for idx, i in enumerate(range(*range_parms)):
+                self[i] = value[idx]
+
+    def __delitem__(self, index):
+        # type: (Any) -> None
+        if not isinstance(index, slice):
+            return self.__delsingleitem__(index)
+        # nprint(index.start, index.stop, index.step, index.indices(len(self)))
+        for i in reversed(range(*index.indices(len(self)))):
+            del self[i]
+
+    @abstractmethod
+    def __getsingleitem__(self, index):
+        # type: (Any) -> Any
+        raise IndexError
+
+    @abstractmethod
+    def __setsingleitem__(self, index, value):
+        # type: (Any, Any) -> None
+        raise IndexError
+
+    @abstractmethod
+    def __delsingleitem__(self, index):
+        # type: (Any) -> None
+        raise IndexError
