@@ -16,13 +16,15 @@ from ruamel_yaml.error import (MarkedYAMLError, MarkedYAMLFutureWarning,
 from ruamel_yaml.nodes import *                               # NOQA
 from ruamel_yaml.nodes import (SequenceNode, MappingNode, ScalarNode)
 from ruamel_yaml.compat import (utf8, builtins_module, to_str, PY2, PY3,  # NOQA
-                                ordereddict, text_type, nprint, version_tnf, Hashable)
+                                ordereddict, text_type, nprint, version_tnf, Hashable,
+                                MutableSequence, MutableMapping)
 from ruamel_yaml.comments import *                               # NOQA
 from ruamel_yaml.comments import (CommentedMap, CommentedOrderedMap, CommentedSet,
-                                  CommentedKeySeq, CommentedSeq, TaggedScalar)
-from ruamel_yaml.scalarstring import *                           # NOQA
-from ruamel_yaml.scalarstring import (PreservedScalarString, SingleQuotedScalarString,
-                                      DoubleQuotedScalarString, ScalarString)
+                                  CommentedKeySeq, CommentedSeq, TaggedScalar,
+                                  CommentedKeyMap)
+from ruamel_yaml.scalarstring import (SingleQuotedScalarString, DoubleQuotedScalarString,
+                                      LiteralScalarString, FoldedScalarString,
+                                      ScalarString,)
 from ruamel_yaml.scalarint import ScalarInt, BinaryInt, OctalInt, HexInt, HexCapsInt
 from ruamel_yaml.scalarfloat import ScalarFloat
 from ruamel_yaml.timestamp import TimeStamp
@@ -75,9 +77,9 @@ class BaseConstructor(object):
         try:
             return self.loader._composer
         except AttributeError:
-            print('slt', type(self))
-            print('slc', self.loader._composer)
-            print(dir(self))
+            sys.stdout.write('slt {}\n'.format(type(self)))
+            sys.stdout.write('slc {}\n'.format(self.loader._composer))
+            sys.stdout.write('{}\n'.format(dir(self)))
             raise
 
     @property
@@ -247,11 +249,45 @@ class BaseConstructor(object):
         # type: (Any, Any, Any, Any, Any) -> None
         if key in mapping:
             if not self.allow_duplicate_keys:
+                mk = mapping.get(key)
+                if PY2:
+                    if isinstance(key, unicode):
+                        key = key.encode('utf-8')
+                    if isinstance(value, unicode):
+                        value = value.encode('utf-8')
+                    if isinstance(mk, unicode):
+                        mk = mk.encode('utf-8')
                 args = [
                     'while constructing a mapping',
                     node.start_mark,
                     'found duplicate key "{}" with value "{}" '
-                    '(original value: "{}")'.format(key, value, mapping.get(key)),
+                    '(original value: "{}")'.format(key, value, mk),
+                    key_node.start_mark,
+                    """
+                    To suppress this check see:
+                        http://yaml.readthedocs.io/en/latest/api.html#duplicate-keys
+                    """,
+                    """\
+                    Duplicate keys will become an error in future releases, and are errors
+                    by default when using the new API.
+                    """,
+                ]
+                if self.allow_duplicate_keys is None:
+                    warnings.warn(DuplicateKeyFutureWarning(*args))
+                else:
+                    raise DuplicateKeyError(*args)
+
+    def check_set_key(self, node, key_node, setting, key):
+        # type: (Any, Any, Any, Any, Any) -> None
+        if key in setting:
+            if not self.allow_duplicate_keys:
+                if PY2:
+                    if isinstance(key, unicode):
+                        key = key.encode('utf-8')
+                args = [
+                    'while constructing a set',
+                    node.start_mark,
+                    'found duplicate key "{}"'.format(key),
                     key_node.start_mark,
                     """
                     To suppress this check see:
@@ -398,9 +434,9 @@ class SafeConstructor(BaseConstructor):
             return sign * int(value_s[2:], 16)
         elif value_s.startswith('0o'):
             return sign * int(value_s[2:], 8)
-        elif self.resolver.processing_version != (1, 2) and value_s[0] == '0':
+        elif self.resolver.processing_version == (1, 1) and value_s[0] == '0':
             return sign * int(value_s, 8)
-        elif self.resolver.processing_version != (1, 2) and ':' in value_s:
+        elif self.resolver.processing_version == (1, 1) and ':' in value_s:
             digits = [int(part) for part in value_s.split(':')]
             digits.reverse()
             base = 1
@@ -774,7 +810,8 @@ class Constructor(SafeConstructor):
             )
         if u'.' in name:
             lname = name.split('.')
-            lmodule_name, lobject_name = lname, []
+            lmodule_name = lname
+            lobject_name = []  # type: List[Any]
             while len(lmodule_name) > 1:
                 lobject_name.insert(0, lmodule_name.pop())
                 module_name = '.'.join(lmodule_name)
@@ -786,7 +823,7 @@ class Constructor(SafeConstructor):
                     continue
         else:
             module_name = builtins_module
-            object_name = [name]
+            lobject_name = [name]
         try:
             __import__(module_name)
         except ImportError as exc:
@@ -994,10 +1031,24 @@ class RoundTripConstructor(SafeConstructor):
             )
 
         if node.style == '|' and isinstance(node.value, text_type):
-            pss = PreservedScalarString(node.value)
+            lss = LiteralScalarString(node.value)
             if node.comment and node.comment[1]:
-                pss.comment = node.comment[1][0]  # type: ignore
-            return pss
+                lss.comment = node.comment[1][0]  # type: ignore
+            return lss
+        if node.style == '>' and isinstance(node.value, text_type):
+            fold_positions = []  # type: List[int]
+            idx = -1
+            while True:
+                idx = node.value.find('\a', idx + 1)
+                if idx < 0:
+                    break
+                fold_positions.append(idx - len(fold_positions))
+            fss = FoldedScalarString(node.value.replace('\a', ''))
+            if node.comment and node.comment[1]:
+                fss.comment = node.comment[1][0]  # type: ignore
+            if fold_positions:
+                fss.fold_pos = fold_positions  # type: ignore
+            return fss
         elif bool(self._preserve_quotes) and isinstance(node.value, text_type):
             if node.style == "'":
                 return SingleQuotedScalarString(node.value)
@@ -1138,7 +1189,7 @@ class RoundTripConstructor(SafeConstructor):
                 width -= 1
             e_width = len(exponent)
             e_sign = exponent[0] in '+-'
-            # print('sf', width, prec, m_sign, exp, e_width, e_sign)
+            # nprint('sf', width, prec, m_sign, exp, e_width, e_sign)
             return ScalarFloat(  # type: ignore
                 sign * float(value_s),
                 width=width,
@@ -1264,7 +1315,7 @@ class RoundTripConstructor(SafeConstructor):
         # type: () -> None
         pass
 
-    def construct_mapping(self, node, maptyp=None, deep=False):  # type: ignore
+    def construct_mapping(self, node, maptyp, deep=False):  # type: ignore
         # type: (Any, Any, bool) -> Any
         if not isinstance(node, MappingNode):
             raise ConstructorError(
@@ -1287,14 +1338,20 @@ class RoundTripConstructor(SafeConstructor):
             key = self.construct_object(key_node, deep=True)
             # lists are not hashable, but tuples are
             if not isinstance(key, Hashable):
-                if isinstance(key, list):
-                    key_a = CommentedKeySeq(key)
+                if isinstance(key, MutableSequence):
+                    key_s = CommentedKeySeq(key)
                     if key_node.flow_style is True:
-                        key_a.fa.set_flow_style()
+                        key_s.fa.set_flow_style()
                     elif key_node.flow_style is False:
-                        key_a.fa.set_block_style()
-                    key = key_a
-                    # key = tuple(key)
+                        key_s.fa.set_block_style()
+                    key = key_s
+                elif isinstance(key, MutableMapping):
+                    key_m = CommentedKeyMap(key)
+                    if key_node.flow_style is True:
+                        key_m.fa.set_flow_style()
+                    elif key_node.flow_style is False:
+                        key_m.fa.set_block_style()
+                    key = key_m
             if PY2:
                 try:
                     hash(key)
@@ -1386,7 +1443,7 @@ class RoundTripConstructor(SafeConstructor):
                     )
             # construct but should be null
             value = self.construct_object(value_node, deep=deep)  # NOQA
-            self.check_mapping_key(node, key_node, typ, key, value)
+            self.check_set_key(node, key_node, typ, key)
             if key_node.comment:
                 typ._yaml_add_comment(key_node.comment, key=key)
             if value_node.comment:
